<?php

namespace MGModule\AdvancedBilling\extensions\ClientAreaUsageRecords;

/**
 * Display usage records in client area.
 */
class ClientAreaUsageRecords extends \MGModule\AdvancedBilling\core\extension\Extension implements \MGModule\AdvancedBilling\core\events\WHMCSBasedEvents
{
    const DISPLAY_ALL = 0;
    const DISPLAY_CURRENT_MONTH = 1;
    const DISPLAY_FROM_LAST_WEEK = 2;
    const DISPLAY_FROM_LAST_DAY = 3;
    const DISPLAY_FROM_LAST_HOUR = 4;
    public $type = self::TYPE_CLIENT_AREA;
    public $name = 'ClientAreaUsageRecords';
    public $friendlyName = 'Usage Records';
    public $description = 'Extension used to view details of usage records and their history per product';
    public $version = '1.0.1';

    public function __construct()
    {
        parent::__construct();
        $obj = new \MGModule\AdvancedBilling\core\form\fields\Switcher('enable', 'enable', 'Enable', 'Check this option if you want to enable usage records extension for this product', false);
        $obj->addStyle('colWidth', 9);
        $this->productConfiguration->add($obj);
        $obj = new \MGModule\AdvancedBilling\core\form\fields\Select2('displayForm', 'displayForm', 'Display Summary From', '', self::DISPLAY_ALL, self::DISPLAY_ALL, [self::DISPLAY_ALL => 'All Time', self::DISPLAY_CURRENT_MONTH => 'Current Month', self::DISPLAY_FROM_LAST_WEEK => 'Last Week', self::DISPLAY_FROM_LAST_DAY => 'Last Day', self::DISPLAY_FROM_LAST_HOUR => 'Last Hour']);
        $obj->addStyle('colWidth', 9);
        $this->productConfiguration->add($obj);
        $obj = new \MGModule\AdvancedBilling\core\form\fields\Select2('usageRecordsPrecision', 'usageRecordsPrecision', 'Usage Records Precision', '', 2, 2, [0, 1, 2, 3, 4, 5]);
        $obj->addStyle('colWidth', 9);
        $this->productConfiguration->add($obj);
        $obj = new \MGModule\AdvancedBilling\core\form\fields\Switcher('showUsageRecords', 'showUsageRecords', 'Enable Records History', '', false);
        $obj->addStyle('colWidth', 9);
        $this->productConfiguration->add($obj);
        $obj = new \MGModule\AdvancedBilling\core\form\fields\Select2('usageRecordsPerPage', 'usageRecordsPerPage', 'Usage Records Per Page', '', 10, 10, [10 => 10, 20 => 20, 50 => 50, 100 => 100]);
        $obj->addStyle('colWidth', 9);
        $this->productConfiguration->add($obj);
        $obj = new \MGModule\AdvancedBilling\core\form\fields\Text('extension', 'extension', '', '', $this->name);
        $obj->addStyle('additionalClass', 'hidden');
        $this->extensionConfiguration->add($obj);
        $license = new \MGModule\AdvancedBilling\core\form\fields\Text('license', 'license', 'License Key');
        $license->addStyle('colWidth', 9)->addStyle('custom', 'width: 100%');
        $this->extensionConfiguration->add($license);
    }

    /**
     * This function will be called from client area.
     *
     * @global type $currency
     *
     * @param type $hostingId
     * @param type $productId
     *
     * @return bool
     */
    public function integrationCodeShow($hostingId, $productId)
    {
        $product = new \MGModule\AdvancedBilling\classes\Product($productId);
        $pricing = $product->getResourcePricing();
        if (empty(array_values($pricing)[0])) {
            return null;
        }

        if (strpos($_SERVER['SCRIPT_NAME'], 'cart.php') !== false) {
            $userId = $_REQUEST['clientsdetails']['userid'];
            $product = new \MGModule\AdvancedBilling\classes\Product($productId);
            $resourcesSettings = $product->getResourceSettings();
            $vars['resources'] = $resourcesSettings;
            $vars['currency'] = getCurrency($userId);
            \MGModule\AdvancedBilling\mgLibs\Lang::setContext('ClientAreaUsageRecords');
            \MGModule\AdvancedBilling\mgLibs\Lang::loadLangFromFile(__DIR__ .DS.'lang');
            \MGModule\AdvancedBilling\mgLibs\Smarty::I()->setTemplateDir(__DIR__);

            return \MGModule\AdvancedBilling\mgLibs\Smarty::I()->view('templates/productConfig', $vars);
        }

        if ((strpos($_SERVER['SCRIPT_NAME'], 'clientarea.php') !== false) && ($_REQUEST['action'] == 'productdetails' )) {
            $this->loadProductConfiguration($productId);
            if (!$this->productConfiguration->get('enable')->value) {
                return false;
            }

            $startDate = null;
            $endDate = null;
            switch ($this->productConfiguration->get('displayForm')->value) {
                case self::DISPLAY_CURRENT_MONTH:
                    $startDate = date('Y-m-01 00:00:00');
                    $endDate = date('Y-m-t 23:59:59');

                    break;
                case self::DISPLAY_FROM_LAST_WEEK:
                    $startDate = date('Y-m-d 00:00:00', strtotime('-7 DAYS'));
                    $endDate = date('Y-m-d 23:59:59');

                    break;
                case self::DISPLAY_FROM_LAST_DAY:
                    $startDate = date('Y-m-d 00:00:00', strtotime('-1 DAY'));
                    $endDate = date('Y-m-d 23:59:59');

                    break;
                case self::DISPLAY_FROM_LAST_HOUR:
                    $startDate = date('Y-m-d h:i:s', strtotime('-1 hour'));
                    $endDate = date('Y-m-d h:i:s');

                    break;
            }
            $precision = $this->productConfiguration->get('usageRecordsPrecision')->value;
            $resources = $this->getResourcesSummary($productId, $hostingId, $precision);
            $records = [];
            $showHistory = $this->productConfiguration->get('showUsageRecords')->value;
            $page = (isset($_REQUEST['ABRecordsPage']) ? (int) ($_REQUEST['ABRecordsPage']) : 1);
            $nextPage = '';
            $prevPage = '';
            $pagesCount = '';
            if ($resources && $showHistory) {
                $account = new \MGModule\AdvancedBilling\classes\Account($hostingId);
                $limit = (int) $this->productConfiguration->get('usageRecordsPerPage')->value;
                $records = $this->getHistoryData($page, $limit, $account, $resources, $precision);
                $count = $account->getItemsAmount();
                if (1 < $page) {
                    $prevPage = $page - 1;
                }

                $pagesCount = ceil($count / $limit);
                if ($page < $pagesCount) {
                    $nextPage = $page + 1;
                }
            }

            \MGModule\AdvancedBilling\mgLibs\Lang::setContext('ClientAreaUsageRecords');
            \MGModule\AdvancedBilling\mgLibs\Lang::loadLangFromFile(__DIR__.DS.'lang');
            $vars = ['id' => $hostingId, 'resources' => $resources, 'records' => $records, 'ABRecordsPage' => $page, 'nextPage' => $nextPage, 'prevPage' => $prevPage, 'pagesCount' => $pagesCount, 'showHistory' => $showHistory, 'columnCount' => 2 + count($resources)];
            \MGModule\AdvancedBilling\mgLibs\Smarty::I()->setTemplateDir(__DIR__);

            return \MGModule\AdvancedBilling\mgLibs\Smarty::I()->view('templates/productDetails', $vars);
        }
    }

    public function checkLicense()
    {
        $key = $this->extensionConfiguration->get('license')->value;
        $file = ROOTDIR.\DIRECTORY_SEPARATOR.'modules'.\DIRECTORY_SEPARATOR.'addons'.\DIRECTORY_SEPARATOR.'AdvancedBilling'.\DIRECTORY_SEPARATOR.'license.php';
        if (!$key && file_exists($file)) {
            include $file;
            $this->extensionConfiguration->set('license', $advanced_billing_licensekey);
            $this->extensionConfiguration->save();
            $key = $advanced_billing_licensekey;
        }

        global $CONFIG;
        $moduleVersionFile = __DIR__ . '/moduleVersion.php';
        $moduleVersion = '';
        if (file_exists($moduleVersionFile)) {
            require $moduleVersionFile;
        }

        $whmcsVersion = $CONFIG['Version'];
        $results = ['status' => 'Unknown Error', 'description' => ''];
       
$results['status'] = 'Active';
$results['description'] = 'Your module license is active.';
$license_check['status'] = 'Active';
$license_check['description'] = 'Your module license is active.';
        $license_check = $results;
       if ($results['status'] != 'Active') {
            $error_message = (isset($license_custom_error_message) ? $license_custom_error_message : 'License '.$license_check['status'].(($license_check['description'] ? ': '.$license_check['description'] : '')));

            throw new \Exception($error_message);
        }
    }

    protected function activate()
    {
    }

    protected function deactivate()
    {
    }

    private function getResourcesSummary($productId, $hostingId, $precision)
    {
        $account = new \MGModule\AdvancedBilling\classes\Account($hostingId);
        $summary = $account->getSummary();
        $product = new \MGModule\AdvancedBilling\classes\Product($productId);
        $resources = $product->getResourceSettings();
        $currency = getCurrency($account->client->id);

        return $this->createSummary($summary, $resources, $currency, $precision);
    }

    private function createSummary($summary, $resources, $currency, $precision)
    {
        foreach ($resources as $resource) {
            if ('on' === $resource['pricing']['enabled']) {
                $result[$resource['name']] = ['friendlyName' => $resource['friendlyName'], 'price' => $currency['prefix'].round($summary['total'][$resource['name']]['price'], $precision).$currency['suffix'], 'usage' => round($summary['total'][$resource['name']]['usage'], $precision), 'unit' => $resource['pricing']['unit']];
            }
        }

        return $result;
    }

    private function getHistoryData($page, $limit, $account, $resources, $precision)
    {
        $currency = getCurrency($account->client->id);
        $items = $account->getMixedItems($page * $limit, ($page - 1) * $limit);
        $records = [];
        $this->mergeHistoryRecords($records, $items['fresh'], $currency, $precision, $resources);
        $this->mergeHistoryRecords($records, $items['hourly'], $currency, $precision, $resources);

        return $records;
    }

    private function mergeHistoryRecords(&$records, $items, $currency, $precision, $resources)
    {
        foreach ($items as $key => $row) {
            foreach ($row as $resName => $res) {
                if (array_key_exists($resName, $resources)) {
                    $records[$key]['items'][$resName]['usage'] = $res;
                    $records[$key]['items'][$resName]['price'] = $currency['prefix'].round($row->{$resName.'Price'}, $precision).$currency['suffix'];
                    $records[$key]['time'] = $row->time;
                    $records[$key]['total'] += $row->{$resName.'Price'};
                }
            }
            $records[$key]['total'] = $currency['prefix'].round($records[$key]['total'], $precision).$currency['suffix'];
        }
    }
}

?>