<?php

namespace MGModule\AdvancedBilling\extensions\Notifications;

/**
 * Display usage records in client area.
 */
class Notifications extends \MGModule\AdvancedBilling\core\extension\Extension implements \MGModule\AdvancedBilling\core\events\WHMCSBasedEvents
{
    public $type = self::TYPE_CLIENT_AREA;
    public $name = 'Notifications';
    public $friendlyName = 'Notifications';
    public $description = 'Extension used to send email reminders to clients when specified resource limits are reached or exceeded';
    public $version = '1.0.0';

    public function __construct()
    {
        parent::__construct();
        $obj = new \MGModule\AdvancedBilling\core\form\fields\Switcher('enable', 'enable', 'Enable', 'Check this option if you want to enable notifications extension for this product', false);
        $obj->addStyle('colWidth', 9);
        $this->productConfiguration->add($obj);
        $emailTemplates = $this->getEmailTemplates();
        asort($emailTemplates);
        $defaultValue = $this->getEmailTemplateDefult();
        $obj = new \MGModule\AdvancedBilling\core\form\fields\Select('emailTemplate', 'emailTemplate', 'Email Template', 'The email template that will be used to send a notification to a client', '', $defaultValue, $emailTemplates);
        $obj->addStyle('additionalClass', 'form-inline')->addStyle('colWidth', 9);
        $this->productConfiguration->add($obj);
        $obj = new \MGModule\AdvancedBilling\core\form\fields\Text('extension', 'extension', '', '', $this->name);
        $obj->addStyle('additionalClass', 'hidden');
        $this->extensionConfiguration->add($obj);
        $license = new \MGModule\AdvancedBilling\core\form\fields\Text('license', 'license', 'License Key');
        $license->addStyle('colWidth', 9)->addStyle('custom', 'width: 100%');
        $this->extensionConfiguration->add($license);
    }

    public function integrationCodeShow($hostingId, $productId)
    {
        $this->loadProductConfiguration($productId);
        if (!$this->productConfiguration->get('enable')->value) {
            return false;
        }

       if ((strpos($_SERVER['SCRIPT_NAME'], 'clientarea.php') !== false) && ($_REQUEST['action'] == 'productdetails')) {
            $controller = new controllers\ClientArea();

            return $controller->indexHTML($productId);
        }
    }

    public function cronStart()
    {
    }

    public function cronEnd()
    {
    }

    /**
     * Charge user account.
     *
     * @param type $productId
     *
     * @return bool
     */
    public function cronLoop($productId)
    {
        $hostings = \MGModule\AdvancedBilling\models\AdvancedBilling\Hosting::factory()->select(['id' => null])->where('packageid', '=', $productId)->fetchAll();
        foreach ($hostings as $hosting) {
            $this->mailRemind($hosting->id);
        }
    }

    public function mail($params)
    {
        $template = $this->getEmailTemplate($params['emailTemplateId']);
        if ($this->mailFrequency($params['reminder'])) {
            $result = \MGModule\AdvancedBilling\Addon::localAPICall('sendemail', ['id' => $params['hostingId'], 'messagename' => $template['name'], 'customvars' => ['type' => $params['limiter']['name'], 'usage' => $params['usage_value'], 'unit' => $params['unit'], 'limit' => $params['limiter']['value'], 'domain' => $params['domain'], 'reminder_name' => $params['reminder']['name']]]);
            if ($result['result'] == 'success') {
                $this->saveEmailSent($params['reminder']);
            }
        }
    }

    public function getClassName()
    {
        $path = explode('\\', 'MGModule\\AdvancedBilling\\extensions\\Notifications\\Notifications');

        return array_pop($path);
    }

    public function getFriendlyName(array $limiter, array $resources)
    {
        foreach ($resources as $resource) {
            if ($limiter['type'] === $resource['name']) {
                return $resource['friendlyName'];
            }
        }
    }

    public function mailRemind($hostingId)
    {
        $reminders_out = $this->getReminders();
        $limiters_out = $this->getLimiters();
        $acc = new \MGModule\AdvancedBilling\classes\Account($hostingId);
        $productId = $acc->product->id;
        $domain = $acc->hosting->domain;
        $userId = $acc->client->id;
        $product = new \MGModule\AdvancedBilling\classes\Product($productId);
        $productResources = $product->getSubmoduleResources()->fetchToArray();
        $pricing = $product->getResourcePricing();
        $extensionId = $this->getExtensionId();
        $emailTemplateId = $this->getEmailTemplateId($extensionId, $productId);
        $endDate = date('Y-m-d H:i:s');
        foreach ($reminders_out as $reminder) {
            $startDate = $this->getStartDate($reminder['periodFrequency']);
            $summary = (array) $acc->getSummary($startDate, $endDate)['total'];
            foreach ($limiters_out as $limiter) {
                $limiter['name'] = $this->getFriendlyName($limiter, $productResources);
                if ($reminder['id'] !== $limiter['reminderId'] || $reminder['hostingId'] !== $hostingId) {
                    continue;
                }

                if (($reminder['status'] != 'enabled') || ($limiter['conditional'] == null) || ($limiter['value'] == 0)) {
                    continue;
                }

                $usage_value = $summary[$limiter['type']]['usage'];
                $condition = $limiter['conditional'];
                $limiter_value = $limiter['value'];
                if ($this->condition($usage_value, $condition, $limiter_value)) {
                    $data = ['hostingId' => $hostingId, 'limiter' => $limiter, 'usage_value' => $usage_value, 'unit' => str_replace('/hr', '', $pricing[$limiter['type']]['unit']), 'domain' => $domain, 'reminder' => $reminder, 'emailTemplateId' => $emailTemplateId];
                    $this->mail($data);
                }
            }
        }
    }

    public function saveEmailSent(array $reminder)
    {
        $now = date('Y-m-d H:i:s');
        $reminderDateUpdate = models\Reminder::factory($reminder['id']);
        $reminderDateUpdate->lastMailSentDate = $now;
        $reminderDateUpdate->save();

        return true;
    }

    public function mailFrequency(array $reminder)
    {
        if ($reminder['lastMailSentDate'] != null) {
            $mailSentDate = $reminder['lastMailSentDate'];
            $now = date('Y-m-d H:i:s');
            $timestampNow = strtotime($now);
            $timestampMailSentDate = strtotime($mailSentDate);
            $timestampMailFrequency = $reminder['mailFrequency'] * 3600;
            if ($timestampNow - $timestampMailSentDate < $timestampMailFrequency) {
                return false;
            }
        }

        return true;
    }

    public function getStartDate($datePeriod)
    {
        $now = date('Y-m-d H:i:s');
        $timestamp = strtotime($now);
        $user_time_frequencyInSeconds = $this->periodToHourConverter($datePeriod) * 3600;
        $timestamp = $timestamp - $user_time_frequencyInSeconds;

        return date('Y-m-d H:i:s', $timestamp);
    }

    public function condition($usage_value, $condition, $limiter_value)
    {
        if ($limiter_value != 0) {
            switch ($condition) {
                case '=':
                    if ($usage_value === $limiter_value) {
                        return true;
                    }

                    // no break
                case '>=':
                    if ($limiter_value <= $usage_value) {
                        return true;
                    }

                    // no break
                case '>':
                    if ($limiter_value < $usage_value) {
                        return true;
                    }

                    // no break
                default:
                    return false;
            }
        } else {
            return false;
        }
    }

    public function periodToHourConverter($period)
    {
        switch ($period) {
            case '1 Hour':
                return 1;
            case '3 Hours':
                return 3;
            case '6 Hours':
                return 6;
            case '1 Day':
                return 24;
            case '3 Days':
                return 72;
            case '1 Week':
                return 168;
            case '2 Weeks':
                return 336;
        }

        return $period;
    }

    public function getReminders()
    {
        $reminder_factory = models\Reminder::factory()->fetchAll()->fetchToArray();
        $reminders_out = [];
        foreach ($reminder_factory as $reminder) {
            $reminders_out[] = (array) $reminder;
        }

        return $reminders_out;
    }

    public function getLimiters()
    {
        $limiters = models\Limiter::factory()->fetchAll();
        $limiters_out = [];
        foreach ($limiters as $limiter) {
            $limiters_out[] = (array) $limiter;
        }

        return $limiters_out;
    }

    public function checkLicense()
    {
        $key = $this->extensionConfiguration->get('license')->value;
        $file = ROOTDIR.\DIRECTORY_SEPARATOR.'modules'.\DIRECTORY_SEPARATOR.'addons'.\DIRECTORY_SEPARATOR.'AdvancedBilling'.\DIRECTORY_SEPARATOR.'license.php';
        if (!$key && file_exists($file)) {
            include $file;
            $this->extensionConfiguration->set('license', $advanced_billing_licensekey);
            $this->extensionConfiguration->save();
            $key = $advanced_billing_licensekey;
        }

        global $CONFIG;
        $moduleVersionFile = __DIR__ . '/moduleVersion.php';
        $moduleVersion = '';
        if (file_exists($moduleVersionFile)) {
            require $moduleVersionFile;
        }

        $whmcsVersion = $CONFIG['Version'];
        $results = ['status' => 'Unknown Error', 'description' => ''];
       
$results['status'] = 'Active';
$results['description'] = 'Your module license is active.';
$license_check['status'] = 'Active';
$license_check['description'] = 'Your module license is active.';
        $license_check = $results;
        if ($results['status'] != 'Active') {
            $error_message = (isset($license_custom_error_message) ? $license_custom_error_message : 'License '.$license_check['status'].(($license_check['description'] ? ': '.$license_check['description'] : '')));

            throw new \Exception($error_message);
        }
    }

    protected function getEmailTemplateDefult()
    {
        \MGModule\AdvancedBilling\mgLibs\MySQL\Query::useCurrentConnection();
        $defaultEmailTemplateName = \MGModule\AdvancedBilling\Configuration::getDefaultTemplateName();
        $templates = \MGModule\AdvancedBilling\mgLibs\MySQL\Query::select(['id'], 'tblemailtemplates', ['name' => $defaultEmailTemplateName])->fetch();

        return $templates['id'];
    }

    protected function getEmailTemplate($templateId)
    {
        $emailTemplate = \MGModule\AdvancedBilling\mgLibs\MySQL\Query::select(['name'], 'tblemailtemplates', ['id' => $templateId])->fetch();
        if (!$emailTemplate) {
            $defaultTemplateName = \MGModule\AdvancedBilling\Configuration::getDefaultTemplateName();
            $emailTemplate = \MGModule\AdvancedBilling\mgLibs\MySQL\Query::select(['name'], 'tblemailtemplates', ['name' => $defaultTemplateName])->fetch();
        }

        return $emailTemplate;
    }

    protected function getEmailTemplates()
    {
        \MGModule\AdvancedBilling\mgLibs\MySQL\Query::useCurrentConnection();
        $templates = \MGModule\AdvancedBilling\mgLibs\MySQL\Query::select(['id', 'name'], 'tblemailtemplates', ['type' => 'product'])->fetchAll();
        $out = [];
        foreach ($templates as $template) {
            $out[$template['id']] = $template['name'];
        }

        return $out;
    }

    protected function activate()
    {
        $limiters = models\Limiter::factory();
        $limiters->installInDatabase();
        $reminders = models\Reminder::factory();
        $reminders->installInDatabase();
        \MGModule\AdvancedBilling\Configuration::checkForEmailExist();
    }

    /**
     * Suspend service.
     */
    protected function deactivate()
    {
    }

    private function getExtensionId()
    {
        $extensions = \MGModule\AdvancedBilling\mgLibs\MySQL\Query::select(['id', 'extension'], 'AdvancedBilling_Extensions')->fetchAll();
        foreach ($extensions as $extension) {
            if ($extension['extension'] === $this->getClassName()) {
                return $extension['id'];
            }
        }

        return false;
    }

    private function getEmailTemplateId($extensionId, $productId)
    {
        $params = ['extensionId' => $extensionId, 'setting' => 'emailTemplate', 'productId' => $productId];
        $result = \MGModule\AdvancedBilling\mgLibs\MySQL\Query::select(['value'], 'AdvancedBilling_ExtensionProductConfiguration', $params)->fetch();

        return $result['value'];
    }
}

?>